package com.mortals.xhx.common.utils;

import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.common.Rest;
import com.mortals.framework.util.DataUtil;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.matter.model.MatterEntity;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class MatterHtmlParseUtil {

    public static Rest<Map<String, Integer>> statSiteMatterCount(Map<String, String> params, String url) {
        String matterTotalExp = "//input[@id=\"result_count\"]";
        String matterPageExp = "//input[@id=\"pageNum\"]";
        Map<String, Integer> resultMap = new HashMap<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(matterTotalExp);
            if (elements.size() > 0) {
                Integer total = elements.get(0) == null ? 0 : DataUtil.converStr2Int(elements.get(0).attr("value"), 0);
                resultMap.put("total", total);
            }

            elements = dom.selectXpath(matterPageExp);
            if (elements.size() > 0) {
                Integer pageNum = elements.get(0) == null ? 0 : DataUtil.converStr2Int(elements.get(0).attr("value"), 0);
                resultMap.put("pageNum", pageNum);
            }

        } catch (Exception e) {
            log.error("获取事项数量异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(resultMap);
    }

    public static Rest<List<MatterEntity>> getMatterList(Map<String, String> params, String url) {
        String matterListExp = "//div[@class=\"sx_list\"]//span[1]";
        String matterListLiExp = "//div[@class=\"sx_list\"]//li/a[1]";
        List<MatterEntity> matterEntityList = new ArrayList<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            //System.out.println(dom.html());
            Elements elements = dom.selectXpath(matterListExp);

            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                String href = element.firstElementChild().attr("href");

                //element.child()
                if (href.equalsIgnoreCase("javascript:void(0)")) {
                    continue;
                }
                //抓取申请与评价页面地址
                Element nextElementSibling = element.nextElementSibling();

                String evaluationUrl = "";
                String netApplyUrl = "";

                Elements elementsA = nextElementSibling.selectXpath("//a");
                if (elementsA != null) {
                    for (Element tempElement : elementsA) {
                        if (tempElement.text().trim().equals("好差评")) {
                            String onclick = tempElement.attr("onclick");
                            evaluationUrl = StrUtil.subBetween(onclick, "evaluation('", "')");
                        }
                        if (tempElement.text().trim().equals("申请")) {
                            String onclick = tempElement.attr("onclick");
                            List<String> list = ReUtil.findAllGroup0("'(.*?)'", onclick);
                            if (list.size() > 4) {
                                netApplyUrl = StrUtil.subBetween(list.get(3), "'", "'");
                            }
                        }
                    }
                }
                buildMatter(matterEntityList, title, href, evaluationUrl, netApplyUrl);
            }

            elements = dom.selectXpath(matterListLiExp);
            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                String href = element.attr("href");
                //抓取申请与评价页面地址
                String evaluationUrl = "";
                String netApplyUrl = "";
                Element nextElementSibling = element.nextElementSibling();
                if (nextElementSibling != null) {
                    Elements elementsA = nextElementSibling.selectXpath("//a");
                    for (Element tempElement : elementsA) {
                        if ("好差评".equals(tempElement.text().trim())) {
                            String onclick = tempElement.attr("onclick");
                            evaluationUrl = StrUtil.subBetween(onclick, "evaluation('", "')");
                        }
                        if ("申请".equals(tempElement.text().trim())) {
                            String onclick = tempElement.attr("onclick");
                            List<String> list = ReUtil.findAllGroup0("'(.*?)'", onclick);
                            if (list.size() > 4) {
                                netApplyUrl = StrUtil.subBetween(list.get(3), "'", "'");
                            }
                        }
                    }
                }
                buildMatter(matterEntityList, title, href, evaluationUrl, netApplyUrl);
            }

        } catch (Exception e) {
            log.error("获取列表异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(matterEntityList);
    }

    private static void buildMatter(List<MatterEntity> matterEntityList, String title, String href, String evaluationUrl, String netApplyUrl) {
        UrlBuilder builder = UrlBuilder.ofHttp(href, CharsetUtil.CHARSET_UTF_8);
        String itemCode = builder.getQuery().get("itemCode").toString();
        String taskType = builder.getQuery().get("taskType").toString();
        String deptCode = builder.getQuery().get("deptCode").toString();
        String areaCode = builder.getQuery().get("areaCode").toString();

        MatterEntity matterEntity = new MatterEntity();
        matterEntity.initAttrValue();
        matterEntity.setTcode(itemCode);
        matterEntity.setTname(title);
        matterEntity.setDeptCode(deptCode);
        matterEntity.setMatterNo(itemCode);
        matterEntity.setAreaCode(areaCode);
        matterEntity.setMatterName(title);
        matterEntity.setUrl(href);
        matterEntity.setEvaluationUrl(evaluationUrl);
        matterEntity.setNetApplyUrl(evaluationUrl);
        matterEntity.setSource(SourceEnum.政务网.getValue());
        matterEntityList.add(matterEntity);
    }

    public static Rest<Map<String, String>> syncDeptBySiteId(Map<String, String> params, String url) {
        String deptListExp = "//ul[@class='bm-list']//li";
        Map<String, String> map = new HashMap<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(deptListExp);
            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String deptName = element.attr("title");
                String href = element.firstElementChild().attr("href");
                UrlBuilder builder = UrlBuilder.ofHttp(href, CharsetUtil.CHARSET_UTF_8);
                String deptCode = builder.getQuery().get("deptCode").toString();
                map.put(deptCode, deptName);
            }
        } catch (Exception e) {
            return Rest.fail(e.getMessage());
        }

        return Rest.ok("当前站点同步添加部门成功！", map);
    }


    public static void main(String[] args) {
/*        String url = "http://www.sczwfw.gov.cn/jiq/interface/item/tags";
        HashMap<String, String> params = new HashMap<>();
        params.put("dxType", "21");
        params.put("areaCode", "510107000000");
        params.put("deptCode", "");
        params.put("searchtext", "");
        params.put("pageno", "5");
        params.put("taskType", "");
        Rest<List<MatterEntity>> rest = MatterHtmlParseUtil.getMatterList(params, url);

        System.out.println(rest.getData().size());*/

/*        HashMap<String, String> params = new HashMap<>();

        params.put("areaCode", "510116000000");

        MatterHtmlParseUtil.syncDeptBySiteId(params);*/


        // String url = "http://www.sczwfw.gov.cn/jiq/front/item/qlqd";
      /*  String url = "http://www.sczwfw.gov.cn/jiq/interface/item/tags";
        HashMap<String, String> params = new HashMap<>();
        params.put("areaCode", "510110006007");
        params.put("dxType", "56");
        params.put("deptCode", "");
        params.put("searchtext", "");
        params.put("taskType", "");
        params.put("pageno", "1");
        Rest<Map<String, Integer>> rest = MatterHtmlParseUtil.statSiteMatterCount(params, url);
        System.out.println(JSON.toJSONString(rest));

        List<MatterEntity> allList = new ArrayList<>();
        String url1 = "http://www.sczwfw.gov.cn/jiq/interface/item/tags";
        HashMap<String, String> params1 = new HashMap<>();
        params1.put("dxType", "56");
        params1.put("areaCode", "510110006007");
        params1.put("deptCode", "");
        params1.put("searchtext", "");
        params1.put("taskType", "");
        for (int i = 1; i <= rest.getData().get("pageNum"); i++) {
            params1.put("pageno", String.valueOf(i));
            //System.out.println("params:" + JSON.toJSONString(params1));
            Rest<List<MatterEntity>> restList = MatterHtmlParseUtil.getMatterList(params1, url1);
            if (restList.getCode() == YesNoEnum.YES.getValue() && !ObjectUtils.isEmpty(restList.getData())) {
               // System.out.println("pageno:" + i + " res size:" + restList.getData().size());
                allList.addAll(restList.getData());
            }
        }
       // System.out.println("--------------------------------------------project size" + rest.getData().get("total") + "----------" + allList.size());


        for (int i = 0; i < allList.size(); i++) {
            System.out.println(i + "  " + allList.get(i).getMatterName());
        }*/


        // System.out.println(JSON.toJSONString(allList));


        String url = "http://www.sczwfw.gov.cn/jiq/interface/item/tags";
        HashMap<String, String> params = new HashMap<>();
        params.put("areaCode", "510110006007");
        params.put("dxType", "56");
        params.put("deptCode", "");
        params.put("searchtext", "");
        params.put("taskType", "");
        params.put("pageno", "1");
        Rest<Map<String, Integer>> rest = MatterHtmlParseUtil.statSiteMatterCount(params, url);
        System.out.println(JSON.toJSONString(rest));


        List<MatterEntity> allList = new ArrayList<>();
        String url1 = "http://www.sczwfw.gov.cn/jiq/interface/item/tags";
        HashMap<String, String> params1 = new HashMap<>();
        params1.put("dxType", "56");
        params1.put("areaCode", "510110006007");
        params1.put("deptCode", "");
        params1.put("searchtext", "");
        params1.put("taskType", "");
        for (int i = 1; i <= rest.getData().get("pageNum"); i++) {
            params1.put("pageno", String.valueOf(i));
            Rest<List<MatterEntity>> restList = MatterHtmlParseUtil.getMatterList(params1, url1);
            if (restList.getCode() == YesNoEnum.YES.getValue() && !ObjectUtils.isEmpty(restList.getData())) {
                allList.addAll(restList.getData());
            }
        }

//        for (int i = 0; i < allList.size(); i++) {
//            System.out.println(i + "  " + allList.get(i).getMatterName());
//        }


        url = "http://www.sczwfw.gov.cn/jiq/front/channel/deptSwitch";
        params = new HashMap<>();
        params.put("areaCode", "510110006007");

        Rest<Map<String, String>> rest1 = MatterHtmlParseUtil.syncDeptBySiteId(params, url);
        System.out.println(JSON.toJSONString(rest1));


    }

}
