package com.mortals.xhx.module.app.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.ZipUtils;
import com.mortals.xhx.module.app.model.AppEntity;
import com.mortals.xhx.module.app.model.AppVersionQuery;
import com.mortals.xhx.module.app.service.AppService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.utils.EncodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.xhx.module.app.dao.AppVersionDao;
import com.mortals.xhx.module.app.model.AppVersionEntity;
import com.mortals.xhx.module.app.service.AppVersionService;

import java.io.File;
import java.nio.charset.Charset;
import java.util.Date;


/**
 * AppVersionService
 * 自助终端应用版本历史 service实现
 *
 * @author zxfei
 * @date 2022-11-28
 */
@Service("appVersionService")
public class AppVersionServiceImpl extends AbstractCRUDServiceImpl<AppVersionDao, AppVersionEntity, Long> implements AppVersionService {
    @Autowired
    private AppService appService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private UploadService uploadService;


    @Override
    public Rest<String> appDistribute(AppVersionEntity appVersionEntity, Context context) {
        AppEntity appEntity = appService.get(appVersionEntity.getAppId(), context);
        distributeFile(appVersionEntity, appEntity, context);
        return Rest.ok("部署应用成功！");
    }

    private String distributeFile(AppVersionEntity appVersionEntity, AppEntity appEntity, Context context) {
        SiteEntity siteCache = siteService.getCache(appEntity.getSiteId().toString());
        //判断文件是否存在
        String targetFilePath = uploadService.getFilePath(appVersionEntity.getFilePath());
        if (FileUtil.isEmpty(new File(targetFilePath))) {
            throw new AppException("部署文件不存在！");
        }
        if (!FileUtil.getSuffix(appVersionEntity.getFilePath()).equals("zip")) {
            throw new AppException("部署只支持zip文件！");
        }
        //部署路径是否存在 如果不存在 创建目录，
        //文件构成目录  基础path +站点编码+应用编码  /basepath/sitecode/appcode/v2/
        String path = GlobalSysInfo.getParamValue(Constant.APP_BASE_DISTRIBUTE_PATH, "/home/publish/app");
        String siteCode = siteCache.getSiteCode();
        String appcode = appEntity.getAppCode();
        Integer version = appVersionEntity.getVersion();
        String disPath = path + File.separator + siteCode + File.separator + appcode + File.separator + version;
        //判断目标目录是否存在 如果存在 这删除
        if (FileUtil.isDirectory(disPath)) {
            FileUtil.del(disPath);
        }
        String fileEncode ="UTF-8";
        try {
            fileEncode = EncodeUtil.getEncode(targetFilePath,true);
        } catch (Exception e) {
            log.error("异常",e);
        }
       // ZipUtils.unzip(new File(targetFilePath), disPath);
        ZipUtil.unzip(targetFilePath, disPath, Charset.forName(fileEncode));
        return disPath;

    }

    @Override
    public Rest<Void> appUsed(AppVersionEntity appVersionEntity, Context context) {
        //使用 部署
        AppEntity appEntity = appService.get(appVersionEntity.getAppId(), context);
        String disPath = distributeFile(appVersionEntity, appEntity, context);
        //更新app版本使用
        appVersionEntity.setUsed(YesNoEnum.YES.getValue());
        appVersionEntity.setDistributeFilePath(disPath);

        AppVersionEntity model = new AppVersionEntity();
        model.setUsed(YesNoEnum.NO.getValue());
        AppVersionQuery condition = new AppVersionQuery().appId(appVersionEntity.getAppId());
        this.getDao().update(model,condition);
        this.update(appVersionEntity, context);

        //更新当前app
        appEntity.setFileName(appVersionEntity.getFileName());
        appEntity.setFilePath(appVersionEntity.getFilePath());
        appEntity.setDistributeFilePath(appVersionEntity.getDistributeFilePath());
        appEntity.setVersion(appVersionEntity.getVersion());
        appEntity.setDistribute(YesNoEnum.YES.getValue());
        appEntity.setUpdateTime(new Date());
        appService.getAppDao().update(appEntity);
        return Rest.ok("使用应用成功！");
    }
}