package com.mortals.xhx.module.window.service.impl;

import cn.hutool.core.collection.ListUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.util.HttpUtil;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.business.model.BusinessQuery;
import com.mortals.xhx.module.site.model.SiteBusinessEntity;
import com.mortals.xhx.module.site.model.SiteBusinessQuery;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.window.model.WindowEntity;
import com.mortals.xhx.module.window.model.WindowQuery;
import com.mortals.xhx.module.window.service.WindowService;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.window.dao.WindowBusinessDao;
import com.mortals.xhx.module.window.model.WindowBusinessEntity;
import com.mortals.xhx.module.window.service.WindowBusinessService;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.framework.util.HttpUtil.HEADER_CONTENT_TYPE;
import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_PHP_HTTP_URL;

/**
 * WindowBusinessService
 * 窗口业务 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("windowBusinessService")
@Slf4j
public class WindowBusinessServiceImpl extends AbstractCRUDServiceImpl<WindowBusinessDao, WindowBusinessEntity, Long> implements WindowBusinessService {
    @Autowired
    private WindowService windowService;

    @Override
    protected void findAfter(WindowBusinessEntity entity, PageInfo pageInfo, Context context, List<WindowBusinessEntity> list) throws AppException {
        Map<Long, WindowEntity> collect = windowService.findToMap(new WindowQuery(), context);
        Iterator iterator = list.iterator();
        while (iterator.hasNext()) {
            WindowBusinessEntity item = (WindowBusinessEntity) iterator.next();
            if (!ObjectUtils.isEmpty(item.getWindowId()) && !ObjectUtils.isEmpty(collect.get(item.getWindowId()))) {
                item.setFromnum(collect.get(item.getWindowId()).getFromnum());
            } else {
                iterator.remove();
            }
        }
        super.findAfter(entity, pageInfo, context, list);
    }


    /**
     * @param ids
     * @param context
     * @throws AppException
     */
    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        Arrays.asList(ids).forEach(id -> {
            WindowBusinessEntity windowBusinessEntity = this.get(id, context);
            pushChangeMsg(windowBusinessEntity);
        });
        super.removeBefore(ids, context);
    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveAfter(WindowBusinessEntity entity, Context context) throws AppException {
        //通知php更新
        pushChangeMsg(entity);
        super.saveAfter(entity, context);
    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateAfter(WindowBusinessEntity entity, Context context) throws AppException {
        pushChangeMsg(entity);
        super.updateAfter(entity, context);
    }

    private void pushChangeMsg(WindowBusinessEntity entity) {
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://172.15.28.116:8090");
        phpUrl += "/api/window/winNameChange";
        HashMap<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("windowid", entity.getWindowId());
        paramsMap.put("typeinfo", 1);
        String resp = null;
        try {
            Map<String, String> header = new HashMap<>();
            header.put(HEADER_CONTENT_TYPE, "application/json");
            log.info("\n thirdPartyUrl=>{} \n reqbody=>{} ", phpUrl, JSON.toJSONString(paramsMap));
            resp = HttpUtil.doPost(phpUrl, header, JSON.toJSONString(paramsMap, SerializerFeature.WriteMapNullValue));
            log.info("thirdPartyUrl =>resp:{}", resp);
        } catch (Exception e) {
            log.error("异常:", e);
        }
    }
}